<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\ExpImpNfceService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities\Resposta;
use Teknisa\MAN\Utilities;
use \Datetime;
use Zeedhi\Framework\DTO\Response\Message;

use NFePHP\NFe\Common\Standardize;
use NFePHP\Common\Certificate;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;

use \Exception as Exception;

use Teknisa\MAN\Services\TransmissaoNfceNewService;
use Teknisa\MAN\Factories\TransmissaoNFCE;
use Zeedhi\Framework\DependencyInjection\InstanceManager;


class ExpImpNfceController {     
    
   
    private $expImpNfceService, $utilService,  $nfeTools, $transmissaoService, $notaFiscal,$entityManager;
    private $url = "https://rm.teknisa.com/export-xml-nfce/";
    const VERSAO_LAYOUT = '4.00';
    
    
    public static $logMessages = array();
    
    public function __construct (Utilities\Util $utilService, ExpImpNfceService $expImpNfceService, $instanceManager, TransmissaoNfceNewService $transmissaoService, TransmissaoNFCE $notaFiscal,\Doctrine\ORM\EntityManager $entityManager){
        $this->expImpNfceService = $expImpNfceService;
        $this->utilService = $utilService;
        $this->instanceManager = $instanceManager;
        $this->transmissaoService = $transmissaoService;
        $this->notaFiscal = $notaFiscal;
        $this->entityManager = $entityManager;
    }
    
    
    public function findLog(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_LOGARQ, $parametros);
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_logArquivos', $result->getResult()));
    
    }
    public function findRelErroAlerta(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_ERRO_ALERTA, $parametros);
        
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_RelErroAlerta', $result->getResult()));
    
    }
    public function findLogMsg(Request\Filter $request, Response $response) {
        
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_CONSULTA_LOGMSG, $parametros);
        
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce_logMensagens', $result->getResult()));
    
    }
    
   
   
    
    
    private function setVendasNfce ($arrayDados,$conn){
             
                 $cdsVendaXmlNfce     =  $arrayDados['cdsVendaXmlNfce'];     
                 $cdsItensXmlNfce     =  $arrayDados['cdsItensXmlNfce'];     
                 $cdsMeioPgtoXmlNfce  =  $arrayDados['cdsMeioPgtoXmlNfce'];  
                 $stFilial            =  $arrayDados['stFilial'];            
                 $stCdCaixa           =  $arrayDados['stCdCaixa'];           
                 $stEmpresa           =  $arrayDados['stEmpresa'];           
                 $stNomeXml           =  $arrayDados['seFiles'];             
                 $stCdOperador        =  $arrayDados['stCdOperador'];
                 $stTipoEmissao       =  $arrayDados['stTipoEmissao'];
                
              
              //Verifica se a venda já existe no banco
                 $params['CDFILIAL']       =   $stFilial;
                 $params['NRNOTAFISCALCE'] =   $cdsVendaXmlNfce['NRNOTAFISCALCE'];
                 $params['CDSERIENFCE']    =   $cdsVendaXmlNfce['CDSERIENFCE'];
                 $params['DTENTRVENDA']    =   substr($cdsVendaXmlNfce['DTVENDA'],0,10); 
                
                $consultaVendaNFce = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_VENDA_NFCE,$params,$conn)->getResult();
            
                
                if(empty($consultaVendaNFce)){
                    // Verifica o caixa
                     if(empty($stCdCaixa)){
                          $stCdCaixa = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CAIXA,$params,$conn)->getResult();
                      }
                    
                    
                    //Verifica se o consumidor da Venda está cadastrado no banco 
                    $params['NRINSCRCONS'] = $cdsVendaXmlNfce['NRINSCRCONS'];
                    $consultaConsumidor = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CONSUMIDOR,$params,$conn)->getResult();
                   
                    if(empty($consultaConsumidor[0]['CDCLIENTE'])){
                        $consultaConsumidorPadrao = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_CONSUMIDOR_PADRAO,$params,$conn)->getResult();
                        
                        if(!empty($consultaConsumidorPadrao[0]['CDCLIENTE']) && !empty($consultaConsumidorPadrao[0]['CDCCUSCLIE'])){
                            $consumidorPadrao = true;
                            $params['CDCLIENTE']    =   $consultaConsumidorPadrao[0]['CDCLIENTE'];
                            $params['CDCCUSCLIE']   =   $consultaConsumidorPadrao[0]['CDCCUSCLIE'];
                            $params['NMCONSVEND']   =   $consultaConsumidorPadrao[0]['NMCONSUMIDOR'];
                            $params['CDCONSUMIDOR'] =   $consultaConsumidorPadrao[0]['CDCONSUMIDOR'];
                        }else { 
                                
                                $arrayDados = $this->buildRetorno($arrayDados,true,'E','Filial não possui cliente e centro de custo padrão cadastrados.','Verifique o cadastro na tela Parametrização > Vendas > Cliente.','','','');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                        }
                    }else {
                        $consumidorPadrao = false;
                        $params['CDCLIENTE']    =   $consultaConsumidor[0]['CDCLIENTE'];
                        $params['CDCCUSCLIE']   =   $consultaConsumidor[0]['CDCCUSCLIE'];
                        $params['NMCONSVEND']   =   $consultaConsumidor[0]['NMCONSUMIDOR'];
                        $params['CDCONSUMIDOR'] =   $consultaConsumidor[0]['CDCONSUMIDOR'];
                    }
                    
                    //Verifica Abertura
                    $params['CDCAIXA'] = $stCdCaixa;
                    $params['DTVENDA'] = $cdsVendaXmlNfce['DTVENDA'];
                    $consultaAbertura = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_ABERTURA,$params,$conn)->getResult();
                    
                    if(empty($consultaAbertura)){
                        $params['DTABERCAIX']   = substr(strval($cdsVendaXmlNfce['DTVENDA']),0,10) . ' 00:00:01';
                        $params['DTFECHCAIX']   = substr(strval($cdsVendaXmlNfce['DTVENDA']),0,10) . ' 23:59:59';
                        $params['DTMOVTURCAIX'] = substr(strval($cdsVendaXmlNfce['DTVENDA']),0,10);
                        $params['CDOPERADOR']   = $stCdOperador; 
                        $params['DSOBSCAIXA']   = 'VIA XML NFCE';
                        
                        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_INSERT_TURCAIXA,$params,$conn);
                        $stDtAberCaixa = substr(strval($cdsVendaXmlNfce['DTVENDA']),0,10) . ' 00:00:01';
                       
                    }else{
                        $stDtAberCaixa = $consultaAbertura[0]['DTABERCAIX'];
                        }
                        
                    //Verifica Loja
                    $consultaLoja = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_LOJA,$params,$conn)->getResult();
                    
                   
                    //Insere a Venda no Banco
                    $nrSeqVenda = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GERA_NRSEQVENDA,$params,$conn)->getResult();
                    
                    $params['NRSEQVENDA']       = $nrSeqVenda[0]['NRSEQVENDA'];
                    $params['CDEMPRESA']        = $stEmpresa;
                    $params['IDSITUVENDA']      = 'O';
                    $params['DTABERTUR']        = $stDtAberCaixa;
                    $params['DTFECHAVENDA']     = $cdsVendaXmlNfce['DTVENDA'];
                    $params['CDLOJA']           = $consultaLoja[0]['CDLOJA'];
                    $params['DTABERVENDA']      = $cdsVendaXmlNfce['DTVENDA'];
                    $params['CDSERIESAT']       = "";
                    $params['NRACESSONFCE']     = $cdsVendaXmlNfce['NRACESSONFCE'];
                    $params['NRNOTAFISCALCE']   = $cdsVendaXmlNfce['NRNOTAFISCALCE'];
                    $params['NRLANCTONFCE']     = $cdsVendaXmlNfce['NRLANCTONFCE'];
                    $params['IDTPEMISVEND']     = 'N';
                    $params['IDTPAMBNFCE']      = $cdsVendaXmlNfce['IDTPAMBNFCE'] ? $cdsVendaXmlNfce['IDTPAMBNFCE'] : '1';
                    $params['NRINSJURIEST']     = $cdsVendaXmlNfce['NRINSJURFILI'];
                    $params['DTHRPROTOCONFCE']  = $cdsVendaXmlNfce['DTHRPROTOCONFCE'];
                    $params['NRPROTOCOLONFCE']  = $cdsVendaXmlNfce['NRPROTOCOLONFCE'];
                    $params['DSOBSSTATUSNFCE']  = $cdsVendaXmlNfce['DSOBSSTATUSNFCE'];
                    $params['SGESTADO']         = $cdsVendaXmlNfce['SGESTADO'];
                    $params['DSARQXMLNFCE']     = $cdsVendaXmlNfce['DSARQXMLNFCE'];
                   
                   //Executa Insert da VENDA
                 
                    $resultInsertVenda = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDA,$params,$conn);
                    
                    //Insere arquivo XML no banco
                          $paramsArqXmlNFce['CDFILIAL']     = $stFilial;
                          $paramsArqXmlNFce['CDCAIXA']      = $stCdCaixa;
                          $paramsArqXmlNFce['NRSEQVENDA']   = $nrSeqVenda[0]['NRSEQVENDA'];
                          $paramsArqXmlNFce['DSARQXMLNFCE'] = $cdsVendaXmlNfce['DSARQXMLNFCE'];
                            
                    $resultInsertArqXmlNFce = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_ARQXML_NFCE,$paramsArqXmlNFce,$conn);
                    
                    
                    //Insere o Item da Venda no Banco
                    
                        for ($i = 0; $i <= count($cdsItensXmlNfce['CDPRODUTO'])- 1; $i++){
                           
                           //Verifica se o produto existe no sistema
                           $params['CDBARPRODUTO'] = $cdsItensXmlNfce['CDPRODUTO'][$i];
                           $consultaCdBarProduto =  $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFICA_CODBAR,$params,$conn)->getResult();
                            
                           if(empty($consultaCdBarProduto)){
                            $param['CDBARRAPROD'] =  $cdsItensXmlNfce['CDPRODUTO'][$i];
                            $consultaCdBarProdTab =  $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_CODBAR_PROD_TAB,$params,$conn)->getResult();
                            
                                if(empty($consultaCdBarProdTab)){
                                    $stCdProduto = '';
                                }else{
                                    $stCdProduto   = $consultaCdBarProdTab[0]['CDARVPROD'];
                                    $stNomeProduto = $consultaCdBarProdTab[0]['NMPRODUTO'];
                                }
                           }else{
                               $stCdArvProduto = $consultaCdBarProduto[0]['CDARVPROD'];
                               $stCdProduto = $consultaCdBarProduto[0]['CDPRODUTO'];
                               $stNomeProduto =  $consultaCdBarProduto[0]['NMPRODUTO'];
                           }
                           
                           
                           if(empty($stCdProduto)){
                                $arrayDados = $this->buildRetorno($arrayDados,true,'E','Produto não cadastrado no banco de dados.','Verifique o produto no arquivo XML.','','','');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados);
                                return $arrayDados;
                           }
                           
                           //verifica aliquiota do produto
                           $params['CDPRODUTO'] = $stCdProduto;
                           $consultaAliq = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIF_ALIQ,$params,$conn)->getResult();
                           if(empty($consultaAliq)){
                                $arrayDados = $this->buildRetorno($arrayDados,true,'E','Produto "'.$stCdArvProduto.'" não possui imposto cadastrado no banco de dados.','Verifique o cadastro na tela Parametrização > Vendas > Produto > Imposto do Produto.',$stCdArvProduto,$stNomeProduto,'');
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                           }else{
                               $error   = false;
                               $stError = 'Existem pendências no cadastro de imposto do produto: '. $stCdArvProduto;
                               if(is_null($consultaAliq[0]['CDIMPOSTO'])){
                                   $error   = true;
                                   $stError = $stError . ', Imposto, ';
                               }
                               if(is_null($consultaAliq[0]['VRPEALIMPFIS'])){
                                   $error   = true;
                                   $stError = $stError . ' Alíquota, ';
                               }
                               if(is_null($consultaAliq[0]['CDCFOPPFIS'])){
                                   $error   = true;
                                   $stError = $stError . ' Código CFOP, ';
                               }
                               if(is_null($consultaAliq[0]['CDCSTICMS'])){
                                   $error   = true;
                                   $stError = $stError . ' Código CST (ICMS), ';
                               }
                               if(is_null($consultaAliq[0]['CDCSTPISCOF'])){
                                   $error   = true;
                                   $stError = $stError . ' Código CST (PIS/COFINS), ';
                               }
                               if(is_null($consultaAliq[0]['VRALIQPIS'])){
                                   $error   = true;
                                   $stError = $stError . ' Alíquota PIS, ';
                               }
                               if(is_null($consultaAliq[0]['VRALIQCOFINS'])){
                                   $error   = true;
                                   $stError = $stError . ' Alíquota COFINS. ';
                               }
                              if($error){
                                  
                                $arrayDados = $this->buildRetorno($arrayDados,true,'E',$stError,'Verifique o cadastro na tela Parametrização > Vendas > Produto > Imposto do Produto.',$stCdArvProduto,$stNomeProduto,'');
                                
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                return $arrayDados;
                                   
                              }
                           }
                            
                           $params['NRSEQUITVEND']      = $cdsItensXmlNfce['NRSEQUITVEND'][$i];
                           $params['QTPRODVEND']        = $cdsItensXmlNfce['QTPRODVEND'][$i];
                           $params['VRUNITVEND']        = $cdsItensXmlNfce['VRUNITVEND'][$i];
                           $params['VRUNITVENDCL']      = 0;
                           $params['VRDESITVEND']       = $cdsItensXmlNfce['VRDESITVEND'][$i] ? $cdsItensXmlNfce['VRDESITVEND'][$i] : 0;
                           $params['VRACRITVEND']       = $cdsItensXmlNfce['VRACRITVEND'][$i] ? $cdsItensXmlNfce['VRACRITVEND'][$i] : 0;
                           
                           if($consultaAliq[0]['IDTPIMPOSFIS'] == 'T'){
                               $params['CDORDALIQIMPR'] = '01';
                           }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'F'){
                               $params['CDORDALIQIMPR'] = 'FF';
                           }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'I'){
                               $params['CDORDALIQIMPR'] = 'II';
                           }else if($consultaAliq[0]['IDTPIMPOSFIS'] == 'N'){
                               $params['CDORDALIQIMPR'] = 'NN';
                           }  
                           
                           $params['VRALIQIMPR']        = $consultaAliq[0]['VRPEALIMPFIS'];
                           
                           //Executa Insert itemVenda
                          $resultInsertItemVenda  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_ITEMVENDA,$params,$conn);
                              
                        
                          //Insere ITVENDAIMPOS
                         
                          $vrTotItem        = (floatval($cdsItensXmlNfce['QTPRODVEND'][$i]) * floatval($cdsItensXmlNfce['VRUNITVEND'][$i])) -
                                              (floatval($cdsItensXmlNfce['VRDESITVEND'][$i]) ? floatval($cdsItensXmlNfce['VRDESITVEND'][$i]) : 0) +
                                              (floatval($cdsItensXmlNfce['VRACRITVEND'][$i]) ? floatval($cdsItensXmlNfce['VRACRITVEND'][$i]) : 0);
                           
                         
                           $vrImposto       = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRPEALIMPFIS'])/100,2);
                           $vrImpPis        = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRALIQPIS'])/100,2); 
                           $vrImpCofins     = $this->utilService->roundABNT($vrTotItem * floatval($consultaAliq[0]['VRALIQCOFINS'])/100,2);
                           $vrBaseCalcReduz = 0;
                           $vrImpoProdReduz = 0;
                           $vrBReduzIcms    = 0;
                           
                              $vrBaseCalcReduz = floor(($vrTotItem - ($vrTotItem * floatval($consultaAliq[0]['VRPERCREDUCAO'])/100)) * 100) / 100; // trunca o resultado em 2 casas decimais
                              $vrImpoProdReduz = $this->utilService->roundABNT($vrBaseCalcReduz * floatval($consultaAliq[0]['VRPEALIMPFIS'])/100,2);
                              $vrBReduzIcms    = $vrTotItem - $vrBaseCalcReduz;
                           
                           
                           
                           $paramsVendaImpos['CDFILIAL']              = $stFilial;
                           $paramsVendaImpos['CDCAIXA']               = $stCdCaixa;
                           $paramsVendaImpos['NRSEQVENDA']            = $nrSeqVenda[0]['NRSEQVENDA'];
                           $paramsVendaImpos['NRSEQUITVEND']          = $cdsItensXmlNfce['NRSEQUITVEND'][$i];
                           $paramsVendaImpos['NRSEQITIMPOS']          = '001';
                           $paramsVendaImpos['CDIMPOSTO']             = $consultaAliq[0]['CDIMPOSTO'];
                           $paramsVendaImpos['VRPEALPRODIT']          = floatval($consultaAliq[0]['VRPEALIMPFIS']); 
                           $paramsVendaImpos['VRIMPOPRODIT']          = floatval($vrImposto); 
                           $paramsVendaImpos['VRPERCOFINS']           = floatval($consultaAliq[0]['VRALIQCOFINS']);
                           $paramsVendaImpos['VRPERPIS']              = floatval($consultaAliq[0]['VRALIQPIS']);
                           $paramsVendaImpos['VRPRBCREDUICMS']        = floatval($consultaAliq[0]['VRPERCREDUCAO']);
                           $paramsVendaImpos['VRBASECALCREDUZ']       = floatval($vrBaseCalcReduz);
                           $paramsVendaImpos['VRIMPOPRODREDUZ']       = floatval($vrImpoProdReduz);
                           $paramsVendaImpos['VRBASECALCICMS']        = floatval($vrTotItem);
                           $paramsVendaImpos['VRBCREDUZICMS']         = floatval($vrBReduzIcms); 
                           $paramsVendaImpos['VRBCPISCOFINS']         = floatval($vrTotItem);
                           $paramsVendaImpos['VRIMPPIS']              = floatval($vrImpPis); 
                           $paramsVendaImpos['VRIMPCOFINS']           = floatval($vrImpCofins);           
                           $paramsVendaImpos['CDCSTPRODI']            = $consultaAliq[0]['CDCSTICMS'];
                           $paramsVendaImpos['CDCSTPRODPC']           = $consultaAliq[0]['CDCSTPISCOF'];
                           $paramsVendaImpos['CDCFOPPROD']            = $consultaAliq[0]['CDCFOPPFIS'];
                           
                           
                            
                           // Executa Insert vendaImpos
                          $resultInsertVendaImpos = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_VENDAIMPOS,$paramsVendaImpos,$conn);
                           
                          
                        }
                        
                        
                        //Insere MOVCAIXA
                        
                        for ($i = 0; $i <= count($cdsMeioPgtoXmlNfce['IDPGTO']) - 1; $i++){
                            
                            // valida recebimento
                                
                                $params['DTABERTURACAIXA'] = $params['DTABERTUR'];
                                $nrSeqMovi = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GERA_NRSEQMOVI,$params,$conn)->getResult();
                                
                                if($stTipoEmissao == 'N'){
                                    if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] == '01'){
                                        $stTipoRece = '4'; //Dinheiro
                                    }else if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] == '02'){
                                        $stTipoRece = '3'; //Cheque
                                    }else if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] == '03'){
                                        $stTipoRece = '1'; //Cartão de Crédito
                                    }else if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] == '04'){
                                        $stTipoRece = '2'; //Cartão de Débito
                                    }else if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] == '05'){
                                        $stTipoRece = '9'; //Crédito Pessoal
                                    }else{
                                        $stTipoRece = '4'; //Dinheiro
                                    }
                                }else{
                                    $stTipoRece = '4'; //Dinheiro
                                }
                                
                                $paramsTipoRece['CDFILIAL'] = $stFilial;
                                $paramsTipoRece['CDCAIXA']  = $stCdCaixa;
                                $paramsTipoRece['IDTIPORECE'] = $stTipoRece;
                                
                                $consultaTipoRece = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_VERIFI_TIPORECE,$paramsTipoRece,$conn)->getResult();
                                
                                
                                if(empty($consultaTipoRece)){
                                    $stTipoRece = '001';
                                    $arrayDados = $this->buildRetorno($arrayDados,false,'A','Meio de pagamento "' . $cdsMeioPgtoXmlNfce['IDPGTO'][$i] . '" não encontrado na configuração do caixa. Será cadastrado na venda o recebimento de código 001.','Verifique o cadastro na tela Configuração do Terminal de Caixa > selecione a vigência > Grupos de Recebimento.','','',$stTipoRece);
                                    $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                }else{
                                    $stTipoRece = $consultaTipoRece[0]['CDTIPORECE'];
                                }
                                
                               if($cdsMeioPgtoXmlNfce['VRPGTO'][$i] == 0){
                                    $arrayDados = $this->buildRetorno($arrayDados,false,'A','Meio de pagamento "' . $cdsMeioPgtoXmlNfce['IDPGTO'][$i] . '" possui valor igual a zero no XML. Não será inserido na venda.','Verifique o meio de pagamento no arquivo XML.','','',$stTipoRece);
                               
                                $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                                }else{
                                    $paramsMovCaixa['CDFILIAL']      = $stFilial;
                                    $paramsMovCaixa['CDCAIXA']       = $stCdCaixa;
                                    $paramsMovCaixa['DTABERCAIX']    = $stDtAberCaixa;    
                                    $paramsMovCaixa['NRSEQUMOVI']    = $nrSeqMovi[0]['NRSEQMOVI'];
                                    
                                    if($cdsMeioPgtoXmlNfce['IDPGTO'][$i] != 'Troco'){
                                        $paramsMovCaixa['IDTIPOMOVIVE']  = 'E';  
                                    }else{
                                        $paramsMovCaixa['IDTIPOMOVIVE']  = 'S'; 
                                    }
                                    $paramsMovCaixa['VRMOVIVEND']    = $cdsMeioPgtoXmlNfce['VRPGTO'][$i];  
                                    $paramsMovCaixa['NRSEQVENDA']    = $nrSeqVenda[0]['NRSEQVENDA'];
                                    $paramsMovCaixa['CDTIPORECE']    = $stTipoRece;
                                    $paramsMovCaixa['DTHRINCMOV']    = $cdsVendaXmlNfce['DTVENDA'];
                                    $paramsMovCaixa['DTMOVIMCAIXA']  = substr(strval($cdsVendaXmlNfce['DTVENDA']),0,10);
                                    if($consumidorPadrao){
                                        $paramsMovCaixa['CDCLIENTE'] = $consultaConsumidorPadrao[0]['CDCLIENTE'];
                                    }else{
                                        $paramsMovCaixa['CDCLIENTE'] = $consultaConsumidor[0]['CDCLIENTE'];
                                    }
                                  $resultInsertMovCaixa = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::QUERY_INSERT_MOVCAIXA,$paramsMovCaixa,$conn);
                              
                                }   
                            
                        }
                        
                        if($arrayDados['error']['ERROSTATUS'] === false){
                            $arrayDados = $this->buildRetorno($arrayDados,false,'S','Arquivo importado','','','','');
                           
                            $arrayDados['error']['NRNOTAFISCALCE']   = '';
                            $arrayDados['error']['IDSTATUSIMPARQ']   = 'S';
                            
                            $param['IDSTATUSIMPARQ']  = 'S';
                            $param['NRACESSONFCE']    = $arrayDados['error']['NRACESSONFCE'];
                            $param['NRSEQVENDA']      = $nrSeqVenda[0]['NRSEQVENDA'];
                            
                            $resultUpdateStatusImp  = $this->expImpNfceService->executarQueryInsert(ExpImpNfceService::UPDATE_STATUS_IMP_ARQXML,$param,$conn);
                            
                            return $arrayDados;
                        }else{
                            
                            return $arrayDados;
                        }
                
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,true,'E','Venda já existente no sistema.','','','','');
                    
                    $this->expImpNfceService->insertMsgLogArquivoXml($arrayDados,$conn);
                        
                     return $arrayDados;
                }
           
            
      
    }
    
    private function geraXmlNfce ($loteCFe,$protCFe,$arrayDados,$stDadosFixos,$connection=null){
        
            $infCFe     = $loteCFe->infNFe;
            $ide        = $infCFe->ide;
            $emit       = $infCFe->emit;
            $enderEmit  = $emit->enderEmit;
            $dest       = $infCFe->dest;
            $total      = $infCFe->total;
            $totIcms    = $total->ICMSTot;
            
           
            
            
            
            
            //recebe todo o conteúdo xml do arquivo
            $cdsVendaXmlNfce['DSARQXMLNFCE'] = $stDadosFixos['DSARQXMLNFCE'];
            
            //VENDA
            $cdsVendaXmlNfce['NRACESSONFCE'] = substr($loteCFe->infNFe->attributes()->Id, 3, strlen($loteCFe->infNFe->attributes()->Id));  
            $cdsVendaXmlNfce['NRINSJURFILI'] = strval($emit->CNPJ);
            $cdsVendaXmlNfce['SGESTADO']     = strval($enderEmit->UF);
            $cdsVendaXmlNfce['CDSERIENFCE']  = str_pad($ide->serie,  3, 0,STR_PAD_LEFT); 
            
            
            
            $stData = $ide->dhEmi;
    
            $cdsVendaXmlNfce['DTVENDA']         = date('d/m/Y h:i:s', strtotime($stData));
            $cdsVendaXmlNfce['NRLANCTONFCE']    = substr($stData,8,2) . $ide->cNF;
            $cdsVendaXmlNfce['IDTPAMBNFCE']     = strval($ide->tpAmb);
            $cdsVendaXmlNfce['NRNOTAFISCALCE']  = str_pad($ide->nNF,  9, 0,STR_PAD_LEFT);
            $cdsVendaXmlNfce['NRINSCRCONS']     = strval($dest->CPF);
            
          
            
            $infProt = $protCFe->infProt;
            $stData  = $infProt->dhRecbto;
            $cdsVendaXmlNfce['DTHRPROTOCONFCE'] = date('d/m/Y h:i:s', strtotime($stData));
            $cdsVendaXmlNfce['NRPROTOCOLONFCE'] = strval($infProt->nProt);
            $cdsVendaXmlNfce['DSOBSSTATUSNFCE'] = strval($infProt->cStat . ' - ' . $infProt->xMotivo);
            
            
            
             //ITEMVENDA-ITVENDAIMPOS
             foreach($infCFe->det as $item){
                 $prod = $item->prod;
                 $cdsItensXmlNfce['CDPRODUTO'][]     = strval($prod->cProd);
                 $cdsItensXmlNfce['QTPRODVEND'][]    = floatval($prod->qCom);
                 $cdsItensXmlNfce['VRUNITVEND'][]    = floatval($prod->vUnCom);
                 $cdsItensXmlNfce['NRSEQUITVEND'][]  = str_pad($item->attributes()->nItem,  3, 0,STR_PAD_LEFT);
                 
                
                 if($prod->vDesc){
                     $cdsItensXmlNfce['VRDESITVEND'][] = floatval($prod->cDesc);
                 }else {
                     $cdsItensXmlNfce['VRDESITVEND'][] = 0;
                 }
                 
                 if($prod->vOutro){
                     $cdsItensXmlNfce['VRACRITVEND'][] = floatval($prod->vOutro);
                 }else{
                     $cdsItensXmlNfce['VRACRITVEND'][] = 0;
                 }
             }
             
            
             
             //RECEBIMENTOS
              foreach($infCFe->pag as $pag){
                  if($pag->detPag){
                      $cdsMeioPgtoXmlNfce['IDPGTO'][] = strval($pag->detPag->tPag);
                      $cdsMeioPgtoXmlNfce['VRPGTO'][] = floatval($pag->detPag->vPag);
                  }
                  if($pag->vTroco){
                      $cdsMeioPgtoXmlNfce['IDPGTO'][] = 'Troco';
                      $cdsMeioPgtoXmlNfce['VRPGTO'][] = floatval($pag->vTroco);
                  }
              }
              
                        
              
            // valida se existe filial no sistema usando o CNPJ coletado do  arquivo
                  $params['CDSERIECX'] = $cdsVendaXmlNfce['CDSERIENFCE'];
                  $params['NRINSJURFILI'] = $cdsVendaXmlNfce['NRINSJURFILI'];
                  $consultaFilialNfce  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL_NFCE,$params,$connection)->getResult();
                 
                // se nao existir filial, utiliza os dados de unidade informados
              if(!empty($consultaFilialNfce)){
                  $stEmpresa    = $consultaFilialNfce[0]['CDEMPRESA'];
                  $stCdCaixa    = $consultaFilialNfce[0]['CDCAIXA'];
                  $stFilial     = $consultaFilialNfce[0]['CDFILIAL'];
                  
                }else if(!empty($stDadosFixos['CDFILIAL'])){
                    $consultaFilial  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILIAL,array('CDFILIAL'=>$stDadosFixos['CDFILIAL']),$connection)->getResult();
                    if($consultaFilial[0]['CDFILIAL'] != $stDadosFixos['CDFILIAL']){
                        $arrayDados = $this->buildRetorno($arrayDados,true,'E','Filial informada nao existe no sistema','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (NFC-e).','','','');
                        
                        return $arrayDados;
                    }else{
                        $stEmpresa    = $stDadosFixos['CDEMPRESA'];
                        $stCdCaixa    = $stDadosFixos['CDCAIXA'];
                        $stFilial     = $stDadosFixos['CDFILIAL'];
                        
                    }
                }else{
                    $arrayDados = $this->buildRetorno($arrayDados,true,'E','Filial e caixa não encontrados para o CNPJ e a série do arquivo XML','Informe a filial correta para importação das vendas na tela Importação do Arquivo XML (NFC-e).','','','');
                    
                    return $arrayDados;
                }
                
                
                
              // parametros com dados do arquivo 
              $arrayDados['cdsVendaXmlNfce']     = $cdsVendaXmlNfce;
              $arrayDados['cdsItensXmlNfce']     = $cdsItensXmlNfce;
              $arrayDados['cdsMeioPgtoXmlNfce']  = $cdsMeioPgtoXmlNfce;
              $arrayDados['stFilial']            = $stFilial;
              $arrayDados['stCdCaixa']           = $stCdCaixa;
              $arrayDados['stEmpresa']           = $stEmpresa;
              $arrayDados['stTipoEmissao']       = 'N';
              $arrayDados['seFiles']             = $stDadosFixos['NMARQUIVOXML'];
              $arrayDados['stCdOperador']        = $stDadosFixos['CDOPERADOR'];
             
              
              if(!empty($arrayDados)){
                  return $arrayDados;
              }
         
       
    }
    
    
    private function gravaArqXml($arquivo,$stDadosFixos,$conn=null){
        
        try{    
            if($conn){
                $connection = $conn;
            }else{
        
                $connection = $this->entityManager->getConnection();
            }
                    $connection->beginTransaction();
                   
                    try{
                        $xmlFile = simplexml_load_string($arquivo);
                    }catch(\Exception $e){
                        
                            $arrayDados['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                            $arrayDados['error']['IDSTATUSIMPARQ']   = 'N';
                            $arrayDados = $this->buildRetorno($arrayDados,true,'E','Fomarto do arquivo XML incorreto','','','','');
                            return $arrayDados['error'];
                    }
                    
                    
                    if(@count($xmlFile->NFe)){
                        
                        
                        $arrayDados['stNrOrg']                 = $stDadosFixos['NRORG']; 
                        $arrayDados['error']['ERROSTATUS']     = false;
                        $arrayDados['error']['NMARQUIVOXML']   = $stDadosFixos['NMARQUIVOXML'];
                        $arrayDados['error']['NRACESSONFCE']   = substr($xmlFile->NFe->infNFe->attributes()->Id, 3, strlen($xmlFile->NFe->infNFe->attributes()->Id));
                        $arrayDados['error']['NRNOTAFISCALCE'] = str_pad($xmlFile->NFe->infNFe->ide->nNF,  9, 0,STR_PAD_LEFT);
                        
                        $params['NRACESSONFCE']       = substr($xmlFile->NFe->infNFe->attributes()->Id, 3, strlen($xmlFile->NFe->infNFe->attributes()->Id));
                        $params['DSNOMEARQUIVO']      = $stDadosFixos['NMARQUIVOXML'];           
                        $params['DSARQXMLNFCE']       = $arquivo; 
                        $params['DTENTRVENDA']        = date('d/m/Y', strtotime($xmlFile->NFe->infNFe->ide->dhEmi));     
                        $params['CDFILIAL']           = $stDadosFixos['CDFILIAL'];        
                        $params['CDCAIXA']            = $stDadosFixos['CDCAIXA'];  
                        $params['NRORG']              = $stDadosFixos['NRORG'];
                        $params['CDOPERADOR']         = $stDadosFixos['CDOPERADOR'];
                        $params['NRIMPARQXMLNFCE']    = NULL;
                        $params['IDSTATUSIMPARQ']     = 'N';
                        
                        
                       //Insere os dados do arquivo no banco
                       
                        $result = $this->expImpNfceService->insertLogArquivoXml($params,$connection);
                        
                            $arrayDados['error']['NRIMPARQXMLNFCE'] = $result['P_NRIMPARQXMLNFCE'];
                            $arrayDados['error']['IDSTATUSIMPARQ']  = $result['P_IDSTATUSIMPARQ'];
                            
                        if($result['P_IDSTATUSIMPARQ'] == 'S'){
                            
                            $arrayDados = $this->buildRetorno($arrayDados,true,'E','Arquivo já importado','','','','');
                            $arrayDados['error']['IDSTATUSIMPARQ'] = 'N';
                            
                            return $arrayDados['error'];
                        }
                    
                        
                        $loteCFe = $xmlFile->NFe;
                        $protCFe = $xmlFile->protNFe;
                        
                       
                        
                        $arrayDados = $this->geraXmlNfce($loteCFe,$protCFe,$arrayDados,$stDadosFixos,$connection);
                        
                        
                        if(!empty($arrayDados['error']['ERROSTATUS']) && $arrayDados['error']['ERROSTATUS']){
                            $connection->rollback();
                            return $arrayDados['error'];
                        }
                        
                        $arrayDados = $this->setVendasNfce($arrayDados,$connection);
                        
                        if(!empty($arrayDados['error']['ERROSTATUS']) && $arrayDados['error']['ERROSTATUS']){
                            
                            $connection->rollback();
                            return $arrayDados['error'];
                        }
                        
                        $connection->commit();
                        return $arrayDados['error'];
                        
                    }else{
                            $arrayDados['error']['NMARQUIVOXML']     = $stDadosFixos['NMARQUIVOXML'];
                            $arrayDados = $this->buildRetorno($arrayDados,true,'E','Arquivo Não é do tipo: NFC-e','','','','');
                            return $arrayDados['error'];
                    }
                     
                     
                    
                }catch (\Exception $e){
                    $connection->rollback();
                    throw new Exception($e);
                     
                }
        
    }
    
     public function importarXml (Request\DataSet $request, Response $response) {
         
        
            
            $row = $request->getDataSet()->getRows()[0];
            
             
            $stDadosFixos['CDFILIAL']   = $row['CDFILIAL'];
            $stDadosFixos['CDCAIXA']    = $row['CDCAIXA'];
            $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
            $stDadosFixos['CDOPERADOR'] = $row['CDOPERADOR'];
            
          
            
            //percorre a $row['ARQUIVOXML'] e executa cada arquivo recebido
            foreach ($row['ARQUIVOXML'] as $fileIndex=>$dados){
                
                
                 $b64 = explode(",", $dados['b64File'])[1];
                 $decodedB64 = base64_decode($b64);
                 $stDadosFixos['DSARQXMLNFCE'] = $decodedB64;
                 $stDadosFixos['NMARQUIVOXML'] = $dados['name'];
                 $stDadosFixos['NRORG']              = $row['NRORG'];
                 $stDadosFixos['CDOPERADOR']         = $row['CDOPERADOR'];
                
                $arrayDados[$fileIndex] = $this->gravaArqXml($decodedB64,$stDadosFixos);
                
                
            }
            
           
           
            
         $response->addDataset(new DataSet('retorno', $arrayDados));
       
         
       
     }
     
     public function reimportarXml (Request\DataSet $request, Response $response) {
         
            
            
            $row = $request->getDataSet()->getRows()[0];
            
            
            $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
            $stDadosFixos['CDOPERADOR'] = $row['CDOPERADOR'];
            $stDadosFixos['NRORG']      = $row['NRORG'];
          
           
            //percorre a $row['ARQUIVOXML'] e executa cada arquivo recebido
            foreach ($row['ARQUIVOXML'] as $fileIndex=>$dados){
                $stDadosFixos['CDFILIAL']   = $dados['CDFILIAL'];
                $stDadosFixos['CDCAIXA']    = $dados['CDCAIXA'];
               
                
                 $params = array('NRACESSONFCE'=>$dados['NRACESSONFCE'],
                                     'CDFILIAL'=>$dados['CDFILIAL'],
                                      'CDCAIXA'=>$dados['CDCAIXA']);
                                 
                 $consultaArquivo  = $this->expImpNfceService->executarQuery(ExpImpNfceService::QUERY_GET_FILE,$params)->getResult();
                
                
                 if($consultaArquivo[0]['IDSTATUSIMPARQ'] == 'N'){
                     $arquivo = $consultaArquivo[0]['DSARQXMLNFCE'];
                     
                     $stDadosFixos['DSARQXMLNFCE'] = $arquivo;
                     $stDadosFixos['NMARQUIVOXML'] = $dados['DSNOMEARQUIVO'];
                    
                    $arrayDados[$fileIndex] = $this->gravaArqXml($arquivo,$stDadosFixos);
                    
                 }else{
                    $arrayDados[$fileIndex]['error']['NMARQUIVOXML']    = $dados['DSNOMEARQUIVO'];
                    $arrayDados[$fileIndex]['error']['IDSTATUSIMPARQ'] = 'N';
                    $arrayDados[$fileIndex] = $this->buildRetorno($arrayDados[$fileIndex],true,'E','Arquivo já importado','','','','');
                    $arrayDados[$fileIndex] = $arrayDados[$fileIndex]['error'];
                    
                    
                 }
                
            }
         $response->addDataset(new DataSet('retorno', $arrayDados));
       
         
       
     }
     
     private static function buildRetorno($arrayDados,$erroStatus,$idTipoMsg,$dsErro,$dsAjuda,$cdProduto,$nmProduto,$cdTiporece){
         
            $arrayDados['error']['ERROSTATUS']     = $erroStatus;
            $arrayDados['error']['IDTIPOMSG'][]    = $idTipoMsg;
            $arrayDados['error']['DSERRO'][]       = $dsErro;
            $arrayDados['error']['DSAJUDA'][]      = $dsAjuda;
            $arrayDados['error']['CDPRODUTO'][]      = $cdProduto;
            $arrayDados['error']['NMPRODUTO'][]      = $nmProduto;
            $arrayDados['error']['CDTIPORECE'][]      = $cdTiporece;
            
            return $arrayDados;
     }
     
     
     
    
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function buscar (Request\Filter $request, Response $response) {
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
        $parametros = $this->buildParameters($request);
        $result = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_NOTAS, $parametros);
        if ($result->error())
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        else 
            $response->addDataSet(new DataSet('vnd04400_expimpxmlnfce', $result->getResult()));
    }
    
    public function buildParameters (Request\Filter $request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = array();
        $params[':ALLFIEDLS'] = 'T';
        foreach ($conditions as $filtro) {
            
            if (is_array($filtro['value'])) {
                
                // Especifico desse controller
                if ($filtro['columnName'] == "DTVENDA") {
                    if (!empty($filtro['value'][0]) && !empty($filtro['value'][1])) {
                        $params[':P_DTINI'] = $filtro['value'][0];
                        $params[':P_DTFIM'] = $filtro['value'][1];
                    }
                }
                
                if(strpos($filtro['columnName'], '|') && $filtro['operator'] === 'MAPPED_LIKE_ALL'){
                    $colunas = explode('|', $filtro['columnName']);
                    foreach($colunas as $k => $con){
                        $params[':ALLFIEDLS'] = $filtro['value'][0]['_ALL'];
                    }
                }else{
                    $params[':'.$filtro['columnName']] = implode("_", $filtro['value']);     
                }
            } else {
                $params[':'.$filtro['columnName']] = $filtro['value']; 
            }
        }
        
        // Específico desse controller
        $params = $this->verificarDatas($params);
        $params = $this->formataParametros($params);
        return $params;
    }
    
    public function verificarDatas($parametros) {
        
        if (empty($parametros[':P_DTINI']) || empty($parametros[':P_DTFIM'])) {
            $parametros[':P_DTINI'] = 
            $parametros[':P_DTFIM'] = date('d/m/Y');
        }
        unset($parametros[':DTVENDA']);
        return $parametros;
    }
    
    public function formataParametros($parametros){
        
        if(isset($parametros[':CDFILIAL'])){
            $parametros[':P_CDFILIAL'] = $parametros[':CDFILIAL'];
            unset($parametros[':CDFILIAL']);
        }else{
            $parametros[':P_CDFILIAL'] = '';
        }
        
        if($parametros[':P_CDFILIAL'] == ''){
            $parametros[':P_TCDFILIAL'] = 'T';
        }else{
            $parametros[':P_TCDFILIAL'] = '';
        }
        
        if(isset($parametros[':CDCAIXA'])){
            $parametros[':P_CDCAIXA'] = $parametros[':CDCAIXA'];
            unset($parametros[':CDCAIXA']);
        }else{
            $parametros[':P_CDCAIXA'] = '';
        }
        
        if($parametros[':P_CDCAIXA'] == ''){
            $parametros[':P_TCDCAIXA'] = 'T';
        }else{
            $parametros[':P_TCDCAIXA'] = '';
        }
        
        if(isset($parametros[':SITUACAO'])){
            $parametros[':P_IDSTATUSNFCE'] = $parametros[':SITUACAO'];
            unset($parametros[':SITUACAO']);
        }else{
            $parametros[':P_IDSTATUSNFCE'] = 'A_C';
        }
        
        if(isset($parametros[':NRNOTAFISCALCE'])){
            $parametros[':NRNOTAFISCALCE'] = str_pad($parametros[':NRNOTAFISCALCE'], 9, '0', STR_PAD_LEFT);
        }else{
            $parametros[':NRNOTAFISCALCE'] = 'T';
        }
        
        return $parametros;
    }
    
    public function diretorioXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array(
            'diretorio' => date('Ymd').'-'.uniqid(rand(), false)
        );
        $response->addDataSet(new DataSet('resultado', $arrayCurlFields));
    }
    
    public function formatXML($notas, $HABDESTAG = 'N'){
        if(!empty($notas) && count($notas) > 0){
            foreach ($notas as $key => $value) {
                if(!empty($value['DSARQXMLNFCE'])){
                    
                    $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $value['DSARQXMLNFCE']);
                    $xml = str_replace('<?xml version="1.0"?>', '', $xml);
                    
                    $verificaProtcXML = strpos( $xml, 'nfeProc');
                    $verificaXML = strpos( $xml, 'NFe');
                    
                    if($value['IDTPEMISVEND'] == 'N' && $verificaProtcXML !== false){
                        
                        $notas[$key]['DSARQXMLNFCE'] = $xml;
                        $notas[$key]['NOMEDSARQXMLNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-rec.xml';
                        
                        if($value['IDSTATUSNFCE'] == 'C'){
                            $XCanc = $this->xmlCancelado($value);
                            $notas[$key]['DSARQXMLCANCNFCE'] = $XCanc;
                            $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-can.xml';
                        }
                    }else if($value['IDTPEMISVEND'] == 'N' && $verificaProtcXML === false && $verificaXML !== false) {
                        
                        $tagPos = '';
                        $xmlRetorno = simplexml_load_string($xml);  
                        
                        if(!empty($xmlRetorno)){
                            
                            $arrayData2 = $this->xmlToArray($xmlRetorno);
                            //verifica se XML foi assinado
                            $verificaXMLSign = strpos($xml, 'Signature');
                            if($verificaXMLSign === false){
                            	$xml = $this->signNFCe($value, $xml);
                            	if(!$xml){
                            		//valida os dados do XML
                            		 if(!$this->validaXMLSalvo($arrayData2)){
                            			 //regera XML com os dados de venda atualizado
                            			 $xml = $this->callRemontaXML($value);
                            			 $xml = $this->signNFCe($value, $xml);
                            		 }
                            	}
                            }
                             
                            $tagPos = '</infNFe>';
                            $xmlRetorno = simplexml_load_string($xml);  
                            $arrayData2 = $this->xmlToArray($xmlRetorno);
                            	
                            $arrayDataEnviNFe = array();
                            if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'S'){
                            	$arrayDataEnviNFe['enviNFe'] = $arrayData2['enviNFe'];
                            	$arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                            }else if(isset($arrayData2['enviNFe']) && $HABDESTAG == 'N'){
                            	$arrayDataEnviNFe['enviNFe'] = '';
                            	$arrayData['NFe'] = $arrayData2['enviNFe']['NFe'];
                            	
                            	//retira conforme orientacao do delphi => O xml deve ser gerado a partir da tag <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
                            	$posIn = strpos($xml, '<NFe');
                            	$xmlGerado = substr($xml, $posIn);
                            	$posFi = strpos($xmlGerado, '</enviNFe>');
                            	$xmlGerado = substr($xmlGerado, 0, $posFi);
                            	$xml = $xmlGerado;
                            	
                            }else{
                            	$arrayData = $arrayData2;
                            }
                            
                            if(!empty($value['DTHRPROTOCONFCE'])){
                            	$date = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCONFCE']);
                            	$dataFormatada1 = $date->format('Y-m-d');
                            	$dataFormatada2 = $date->format('H:i:s');
                            	$stFusoHr = '-03:00';
                            	$dhRecbto = $dataFormatada1."T".$dataFormatada2.$stFusoHr;    
                            }else{
                            	$dhRecbto = $arrayData['NFe']['infNFe']['ide']['dhEmi'];    
                            }
                            	
                            $stProtNfe = '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="'.$arrayData['NFe']['infNFe']['@versao'].'">';
                            $stProtNfe .= '<infProt Id="ID'.$value['NRPROTOCOLONFCE'].'">';
                            $stProtNfe .= '<tpAmb>'.$value['IDTPAMBNFCE'].'</tpAmb>';
                            $stProtNfe .= '<verAplic>'.$arrayData['NFe']['infNFe']['@versao'].'</verAplic>';
                            $stProtNfe .= '<chNFe>'.$value['NRACESSONFCE'].'</chNFe>';
                            $stProtNfe .= '<dhRecbto>'.$dhRecbto.'</dhRecbto>';
                            $stProtNfe .= '<nProt>'.$value['NRPROTOCOLONFCE'].'</nProt>';
                            
                            $stProtNfe .= '<digVal>'.$arrayData['NFe']['Signature']['SignedInfo']['Reference']['DigestValue'].'</digVal>';
                            
                            
                            $stProtNfe .= '<cStat>'.$value['CODSTATUS'].'</cStat>';
                            $stProtNfe .= '<xMotivo>'.$value['DSOBSSTATUSNFCE'].'</xMotivo>';
                            $stProtNfe .= '</infProt></protNFe>';
                            
                            $stTextoXML = '';
                            $stTextoXML = '<?xml version="1.0" encoding="UTF-8"?><nfeProc versao="'.$arrayData['NFe']['infNFe']['@versao'].'" xmlns="http://www.portalfiscal.inf.br/nfe">';
                            $stTextoXML = $stTextoXML.$xml.$stProtNfe;
                            $stTextoXML = $stTextoXML.'</nfeProc>';
                            
                            
                            //mudancao pois trara na query principal
                            /*$parametros[':SGESTADO'] = $value['SGESTADO'];
                            $parametros[':IDSERVICO'] = ($value['IDTPEMISVEND'] == 'N') ? 'NFC' : '';
                            $parametros[':IDAMBIENTE'] = $value['IDTPAMBNFCE'];
                            
                            $urlChave = $this->expImpNfceService->executarQuery(ExpImpNfceService::BUSCAR_URLCHAVE, $parametros);
                            if(!empty($urlChave->getResult())){
                            	$url = $urlChave->getResult();
                            	$stUrlChave = '<urlChave>'.$url[0]['WSSITEUFQRCODENFC'].'</urlChave>';
                            }else{
                            	$stUrlChave = '<urlChave> </urlChave>';
                            }*/
                            
                            if(!empty($value['WSSITEUFQRCODENFC'])){
                            	$stUrlChave = '<urlChave>'.$value['WSSITEUFQRCODENFC'].'</urlChave>';
                            }else{
                            	$stUrlChave = '<urlChave> </urlChave>';
                            }
                            
                            if(strpos($stTextoXML, '<infNFeSupl>') == FALSE){
                            	$stQRCode = '<qrCode><![CDATA['.$value['DSQRCODENFCE'].']]></qrCode>';
                            	$stQRCODE = '<infNFeSupl>'.$stQRCode.$stUrlChave.'</infNFeSupl>';
                            	//$posIn = strripos($stTextoXML, $tagPos);
                            	$posFi = strpos($stTextoXML, '<Signature');
                            	$xmlQrCode = substr($stTextoXML, 0, $posFi). $stQRCODE. substr($stTextoXML, $posFi);
                            	$stTextoXML = $xmlQrCode;
                            }
                            
                            $notas[$key]['DSARQXMLNFCE'] = $stTextoXML;
                            $notas[$key]['NOMEDSARQXMLNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-rec.xml';
                            
                            if($value['IDSTATUSNFCE'] == 'C'){
                                $XCanc = $this->xmlCancelado($value);
                                $notas[$key]['DSARQXMLCANCNFCE'] = $XCanc;
                                $notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'NFCe'.$value['NRACESSONFCE'].'-pro-can.xml';
                            }
                        }
                        
                    }else{
                        $notas[$key]['NOMEDSARQXMLNFCE'] = 'AD'.$value['NRACESSONFCE'].'.xml';
                    	if($value['IDSTATUSNFCE'] == 'C'){
                    		$notas[$key]['NOMEDSARQXMLCANCNFCE'] = 'ADC'.$value['NRACESSONFCE'].'.xml';
                    	}
                    }
                }
            }
        }
        return $notas;
    }
    
    private function xmlCancelado($value){
        
    	$xmlCancSalvo = $value['DSARQXMLCANCNFCE'];
    	$xmlCancelamentoSalvo = simplexml_load_string($xmlCancSalvo);
    	
    	$xmlCanc = '<procEventoNFe></procEventoNFe>';
    	$xmlCancelamento = simplexml_load_string($xmlCanc);
    	
    	$xmlCancelamento->addAttribute('versao', '1.00');
    	$xmlCancelamento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    	
    	if(!empty($value['DTHRPROTOCOCANC'])){
    		$dateCan = DateTime::createFromFormat('d/m/Y H:i:s', $value['DTHRPROTOCOCANC']);
    		$dataFormatada1Can = $dateCan->format('Y-m-d');
    		$dataFormatada2Can = $dateCan->format('H:i:s');
    		$stFusoHrCan = '-03:00';
    		$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
    	}else{
    		$dataFormatada1Can = date('Y-m-d');
    		$dataFormatada2Can = date('H:i:s');
    		$stFusoHrCan = '-03:00';
    		$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
    	}
    	
    	if(isset($xmlCancelamentoSalvo->evento)){
    		$evento = $xmlCancelamento->addChild('evento');
    		$evento->addAttribute('versao', '1.00');
    		$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		if($xmlCancelamentoSalvo->evento->infEvento){
    			$infEvento = $evento->addChild('infEvento');
    			
    			$arrayID = $xmlCancelamentoSalvo->evento->infEvento->attributes()->Id;
    			foreach((array)$arrayID as $key => $val){
    				$id = $val;
    			}
    			
    			$infEvento->addAttribute('Id', $id);
    			$infEvento->addChild('cOrgao',$xmlCancelamentoSalvo->evento->infEvento->cOrgao);
    			$infEvento->addChild('tpAmb',$xmlCancelamentoSalvo->evento->infEvento->tpAmb);
    			$infEvento->addChild('CNPJ',$xmlCancelamentoSalvo->evento->infEvento->CNPJ);
    			$infEvento->addChild('chNFe',$xmlCancelamentoSalvo->evento->infEvento->chNFe);
    			$infEvento->addChild('dhEvento',$xmlCancelamentoSalvo->evento->infEvento->dhEvento);
    			$infEvento->addChild('tpEvento',$xmlCancelamentoSalvo->evento->infEvento->tpEvento);
    			$infEvento->addChild('nSeqEvento',$xmlCancelamentoSalvo->evento->infEvento->nSeqEvento);
    			$infEvento->addChild('verEvento',$xmlCancelamentoSalvo->evento->infEvento->verEvento);
    			
    			if($xmlCancelamentoSalvo->evento->infEvento->detEvento){
    				$detEvento = $infEvento->addChild('detEvento');
    				$arrayID = $xmlCancelamentoSalvo->evento->infEvento->detEvento->attributes()->versao;
    				foreach((array)$arrayID as $key => $val2){
    					$versao = $val2;
    				}
    				$detEvento->addAttribute('versao', $versao);
    				$detEvento->addChild('descEvento',$xmlCancelamentoSalvo->evento->infEvento->detEvento->descEvento);
    				$detEvento->addChild('nProt',$xmlCancelamentoSalvo->evento->infEvento->detEvento->nProt);
    				$detEvento->addChild('xJust',$xmlCancelamentoSalvo->evento->infEvento->detEvento->xJust);    
    			}else{
    				$detEvento = $infEvento->addChild('detEvento');
    				$detEvento->addAttribute('versao', '1.00');
    				$detEvento->addChild('descEvento','Cancelamento');
    				$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    				$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);    
    			}
    		}else{
    			$infEvento = $evento->addChild('infEvento');
    			$infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
    			$infEvento->addChild('cOrgao',$value['CDESTADOIBGE']);
    			$infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
    			$infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
    			$infEvento->addChild('chNFe',$value['NRACESSONFCE']);
    			
    			$infEvento->addChild('dhEvento',$dhRecbtoCan);
    			$infEvento->addChild('tpEvento','110111');
    			$infEvento->addChild('nSeqEvento','1');
    			$infEvento->addChild('verEvento','1.00');
    			
    			$detEvento = $infEvento->addChild('detEvento');
    			$detEvento->addAttribute('versao', '1.00');
    			$detEvento->addChild('descEvento','Cancelamento');
    			$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    			$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
    		}
    	}else{
    		$evento = $xmlCancelamento->addChild('evento');
    		$evento->addAttribute('versao', '1.00');
    		$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		
    		$infEvento = $evento->addChild('infEvento');
    		$infEvento->addAttribute('Id', 'ID'.'110111'.$value['NRACESSONFCE'].'01');
    		$infEvento->addChild('cOrgao',$value['CDESTADOIBGE']);
    		$infEvento->addChild('tpAmb',$value['IDTPAMBNFCE']);
    		$infEvento->addChild('CNPJ',$value['NRINSJURIEST']);
    		$infEvento->addChild('chNFe',$value['NRACESSONFCE']);
    		
    		$infEvento->addChild('dhEvento',$dhRecbtoCan);
    		$infEvento->addChild('tpEvento','110111');
    		$infEvento->addChild('nSeqEvento','1');
    		$infEvento->addChild('verEvento','1.00');
    		
    		$detEvento = $infEvento->addChild('detEvento');
    		$detEvento->addAttribute('versao', '1.00');
    		$detEvento->addChild('descEvento','Cancelamento');
    		$detEvento->addChild('nProt',$value['NRPROTOCOLONFCE']);
    		$detEvento->addChild('xJust',$value['DSRAZAOCANCNFCE']);
    	}
    	
    	if(isset($xmlCancelamentoSalvo->retEnvEvento)){
    		if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento)){
    			$retEvento = $xmlCancelamento->addChild('retEvento');
    			$retEvento->addAttribute('versao', '1.00');
    			$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    			if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento)){
    				$infEventoRet = $retEvento->addChild('infEvento');
    				$infEventoRet->addChild('tpAmb',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpAmb);
    				$infEventoRet->addChild('verAplic',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->verAplic);
    				$infEventoRet->addChild('cOrgao',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cOrgao);
    				$infEventoRet->addChild('cStat',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cStat);
    				$infEventoRet->addChild('xMotivo',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xMotivo);
    				$infEventoRet->addChild('chNFe',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->chNFe); 
    				$infEventoRet->addChild('tpEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpEvento);
    				$infEventoRet->addChild('xEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xEvento);
    				$infEventoRet->addChild('nSeqEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nSeqEvento);
    				$infEventoRet->addChild('dhRegEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->dhRegEvento);
    				$infEventoRet->addChild('nProt',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nProt);
    			}else{
    				$infEventoRet = $retEvento->addChild('infEvento');
    				$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    				$infEventoRet->addChild('verAplic','SEFAZ');
    				$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    				$infEventoRet->addChild('cStat','135');
    				$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    				$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    				$infEventoRet->addChild('tpEvento','110111');
    				$infEventoRet->addChild('xEvento','Cancelamento');
    				$infEventoRet->addChild('nSeqEvento','1');
    				$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    				$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    			}
    		}else{
    			$retEvento = $xmlCancelamento->addChild('retEvento');
    			$retEvento->addAttribute('versao', '1.00');
    			$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    			$infEventoRet = $retEvento->addChild('infEvento');
    			$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    			$infEventoRet->addChild('verAplic','SEFAZ');
    			$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    			$infEventoRet->addChild('cStat','135');
    			$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    			$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    			$infEventoRet->addChild('tpEvento','110111');
    			$infEventoRet->addChild('xEvento','Cancelamento');
    			$infEventoRet->addChild('nSeqEvento','1');
    			$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    			$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    		}
    	}else{
    		$retEvento = $xmlCancelamento->addChild('retEvento');
    		$retEvento->addAttribute('versao', '1.00');
    		$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
    		$infEventoRet = $retEvento->addChild('infEvento');
    		$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
    		$infEventoRet->addChild('verAplic','SEFAZ');
    		$infEventoRet->addChild('cOrgao',$value['CDESTADOIBGE']);
    		$infEventoRet->addChild('cStat','135');
    		$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
    		$infEventoRet->addChild('chNFe',$value['NRACESSONFCE']);
    		$infEventoRet->addChild('tpEvento','110111');
    		$infEventoRet->addChild('xEvento','Cancelamento');
    		$infEventoRet->addChild('nSeqEvento','1');
    		$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
    		$infEventoRet->addChild('nProt',$value['NRPROTOCOLOCANC']);
    	}
    	$XCanc = $xmlCancelamento->asXML();
    	return $XCanc;
    }
    
    public function exportarXml(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        //tratamento para habilitar ou desabilitar tags <idLote> e <indSinc>
        if(!isset($row['HABDESTAG'])){
            $row['HABDESTAG'] = 'N';
        }
        //realizar tratamento dos arquivos XMLs
        $notas = $this->formatXML($row['arrRows'], $row['HABDESTAG']);
        
        $arrayCurlFields = array(
            'script' => 'EXP',
            'diretorio' => $row['diretorio'],
            'rows' => $notas
        );
        
        $fields_string = http_build_query($arrayCurlFields);
        if(!empty($fields_string)){ 
           try {
            
                $ch = curl_init();
                curl_setopt( $ch, CURLOPT_URL, $this->url );
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch,CURLOPT_POST, 1);
                curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
                $retorno = json_decode(curl_exec( $ch ));
                $array = (array) $retorno;
                curl_close( $ch );
                
                $response->addDataSet(new DataSet('resultado', $array));
                
            }
            catch(\Exception $e){
                throw new Exception("URL inválida",0,$e);
            }     
        }
        
    }
    
    public function compactarXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'],
            'script' => 'COMP'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            curl_close( $ch );
        }
        catch(\Exception $e){
            throw new Exception("URL inválida",0,$e);
        }
        
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    public function deleteArquivosXML(Request\Row $request, Response $response){
        $row = $request->getRow();
        $retorno = new Resposta();
        
        $arrayCurlFields = array( 
            'diretorio' => $row['diretorio'].'.zip',
            'script' => 'DEL'
        );
        
        try {
            
            $fields_string = http_build_query($arrayCurlFields);
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            $retorno = json_decode(curl_exec( $ch ));
            $array = (array) $retorno;
            curl_close( $ch ); 
        }
        catch(\Exception $e){
            throw new Exception("URL inválida",0,$e);
        }
        
        $response->addDataSet(new DataSet('resultado', $array));
    }
    
    private function xmlToArray($xml, $options = array()) {
        $defaults = array(
            'namespaceSeparator' => ':', // você pode querer que isso seja algo diferente de um cólon
            'attributePrefix' => '@',    // para distinguir entre os nós e os atributos com o mesmo nome
            'alwaysArray' => array(),    // array de tags que devem sempre ser array
            'autoArray' => true,         // só criar arrays para as tags que aparecem mais de uma vez
            'textContent' => '$',        // chave utilizada para o conteúdo do texto de elementos
            'autoText' => true,          // pular chave "textContent" se o nó não tem atributos ou nós filho
            'keySearch' => false,        // pesquisa opcional e substituir na tag e nomes de atributos
            'keyReplace' => false        // substituir valores por valores acima de busca
        );
        $options = array_merge($defaults, $options);
        $namespaces = $xml->getDocNamespaces();
        $namespaces[''] = null; // adiciona namespace base(vazio) 
    
        // Obtém os atributos de todos os namespaces
        $attributesArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->attributes($namespace) as $attributeName => $attribute) {
                // Substituir caracteres no nome do atributo
                if ($options['keySearch']) $attributeName =
                        str_replace($options['keySearch'], $options['keyReplace'], $attributeName);
                $attributeKey = $options['attributePrefix']
                        . ($prefix ? $prefix . $options['namespaceSeparator'] : '')
                        . $attributeName;
                $attributesArray[$attributeKey] = (string)$attribute;
            }
        }
    
        // Obtém nós filhos de todos os namespaces
        $tagsArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->children($namespace) as $childXml) {
                // Recursividade em nós filho
                $childArray = $this->xmlToArray($childXml, $options);
                reset($childArray);
                list($childTagName, $childProperties) = $this->myEach($childArray);
                // Substituir caracteres no nome da tag
                if ($options['keySearch']) $childTagName =
                        str_replace($options['keySearch'], $options['keyReplace'], $childTagName);
                // Adiciona um prefixo namespace, se houver
                if ($prefix) $childTagName = $prefix . $options['namespaceSeparator'] . $childTagName;
    
                if (!isset($tagsArray[$childTagName])) {
                    // Só entra com esta chave
                    // Testa se as tags deste tipo deve ser sempre matrizes, não importa a contagem de elementos
                    $tagsArray[$childTagName] =
                            in_array($childTagName, $options['alwaysArray']) || !$options['autoArray']
                            ? array($childProperties) : $childProperties;
                } elseif (
                    is_array($tagsArray[$childTagName]) && array_keys($tagsArray[$childTagName])
                    === range(0, count($tagsArray[$childTagName]) - 1)
                ) {
                    $tagsArray[$childTagName][] = $childProperties;
                } else {
                    $tagsArray[$childTagName] = array($tagsArray[$childTagName], $childProperties);
                }
            }
        }
    
        // Obtém o texto do nó
        $textContentArray = array();
        $plainText = trim((string)$xml);
        if ($plainText !== '') $textContentArray[$options['textContent']] = $plainText;
    
        $propertiesArray = !$options['autoText'] || $attributesArray || $tagsArray || ($plainText === '')
                ? array_merge($attributesArray, $tagsArray, $textContentArray) : $plainText;
    
        // Retorna o nó como array
        return array(
            $xml->getName() => $propertiesArray
        );
    }
    
    private function myEach(&$arr) {
        $key = key($arr);
        $result = ($key === null) ? false : [$key, current($arr), 'key' => $key, 'value' => current($arr)];
        next($arr);
        return $result;
    }
    
    private function signNFCe($nota, $unsignedXML){
        
		//le certificado
		$certificado = $this->getCertificado($nota);
		//config certificado
		$this->nfeTools = $this->transmissaoService->getNfeTools($nota, $certificado);
		
    	try {
    		
    		$XmlSign = $this->nfeTools->signNFe($unsignedXML);
		    if(!empty($XmlSign)){
				return $XmlSign;
		    }else{
		    	return false;
		    }
			
		} catch (\Exception $e) {
			return false;
		}
    }
    
	
	private function getCertificado($nota){
		
		$params[':CDINSCESTA'] = $nota['CDINSCESTA'];
		$params[':CDFILIAL'] = $nota['CDFILIAL'];
		
		$result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::GET_LINK_CERTIFICADO, $params);
		if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
        }
        
		if (sizeof($result) > 0) {
			if (empty($result[0]['CERTIFICADO'])) {
				throw new \Exception('Erro, certificado não parametrizado. Verifique os parâmetros nas seguintes telas: Empresa -> Inscrição Estadual, e Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT!');
			}
			$link = $result[0]['DSDIRCERTIFICADO'];
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $link);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);
			
			$resultLink = curl_exec($ch);
            $err    = curl_errno($ch);
            $errmsg = curl_error($ch);
			curl_close($ch);
			
			if ($err){
                throw new \Exception("Erro ao obter conteúdo do Certificado Digital: " . $errmsg);
            } 
            
            if (strpos($resultLink, 'Not Found')) {
            	throw new \Exception("Erro 404. Não foi possível recuperar o certificado no servidor.". PHP_EOL . 
            	"Verificar se o Link do Servidor de Imagens encontra-se parametrizado em Paramêtros Gerais => Gerais => Geral." . PHP_EOL . 
            	"Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC-e/SAT e Empresa => Inscrição Estadual" . PHP_EOL .$errmsg);
            }
            
		} else {
			throw new \Exception('Não foi possível recuperar os dados de Inscrição Estadual parametrizada.' . PHP_EOL . 
            	'Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC0e/SAT e Empresa => Inscrição Estadual');
		}
		return $result;
	}
    
    private function validaXMLSalvo($arrayXML){
        
        $valorControle = true;
        foreach($arrayXML as $key => $value){
            //tag NFe
            if(!isset($value['NFe'])){
                $valorControle = false;
            }
            
            //tag infNFe
            if(!isset($value['NFe']['infNFe'])){
                $valorControle = false;
            }
            
            //tag versao
            if(!empty($value['NFe']['infNFe']['@versao'])){
                 $valorControle = false;
            }
            
            //tag IDE
            $ide = $value['infNFe']['ide'];
            if(!empty($ide)){
                foreach($ide as $k => $v){
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cUF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['serie'])){
                         $valorControle = false;
                     }
                     if(!empty($v['nNF'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cMunFG'])){
                         $valorControle = false;
                     }
                     if(!empty($v['cDV'])){
                         $valorControle = false;
                     }
                     if(!empty($v['tpAmb'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
            
            //tag IDE
            $emit = $value['infNFe']['emit'];
            if(!empty($emit)){
                foreach($emit as $k => $v){
                     if(!empty($v['CNPJ'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xNome'])){
                         $valorControle = false;
                     }
                     if(!empty($v['xFant'])){
                         $valorControle = false;
                     }
                     if(!empty($v['IE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CNAE'])){
                         $valorControle = false;
                     }
                     if(!empty($v['CRT'])){
                         $valorControle = false;
                     }
                }   
            }else{
                 $valorControle = false;
            }
        }
        
        return $valorControle;
        
    }
    
    private function callRemontaXML($nota){
	    
        $parametros = array(
            ':CDFILIAL' => $nota['CDFILIAL'],
            ':CDCAIXA' => $nota['CDCAIXA'],
            ':CDLOJA' => $nota['CDLOJA'],
            ':NRSEQVENDA'    => $nota['NRSEQVENDA']
        );
        
        $result = $this->expImpNfceService->executarQuery(TransmissaoNFCE::ADQTRANSMISSAONFCE, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            if(!empty($result->getResult())){
               $resposta = $result->getResult();
               $resposta[0]['CDOPERADOR'] = null;
	    	   $resposta[0]['TOTALVENDA'] = $nota['VRTOTALVENDA'];
	    	   $resposta[0]['DTABERTUR'] = $nota['DTABERTUR'];
               $retornoXML = $this->transmissaoService->montaXMLTransmitir($resposta, false);
               
               if(!$retornoXML['error']){
                   return $retornoXML['xml'];
               }
            }
        }
    }
    
       public function apixml(Request\Row $request, Response $response) {
           $rows = $request->getRow();
            $row = array(
           'CDFILIAL'=>$rows["unidade"],
           'NRORG'=>$rows["nrorg"],
           'CDCAIXA'=>$rows["caixa"],
           'NMARQUIVOXML'=>$rows["arquivo"],
           'DSARQUIVOXML'=>$rows["conteudo"],
           'CDEMPRESA'=>[null]
           
           );
           
           $queryFILIAL = "SELECT CDFILIAL,CDEMPRESA
                    FROM FILIAL
                   WHERE 
                   CDFILIAL=:CDFILIAL";
           $params = array( 'CDFILIAL' => $row['CDFILIAL'] );
           
           $connectionCustom =InstanceManager::getInstance()->getService('connectionCustom');
          
           
           $entityManager = $connectionCustom->updateVpdConnection($row['NRORG']);
           $entityManager = $entityManager ? $entityManager : InstanceManager::getInstance()->getService('entityManager');
           $connection = $entityManager->getConnection();
           $result = $connection->fetchAll($queryFILIAL, $params);
           ///validar se retornou dados////

           if($result){
            
            $row['CDEMPRESA']=$result[0]["CDEMPRESA"];
            $stDadosFixos['CDFILIAL']   = $row['CDFILIAL'];
            $stDadosFixos['CDCAIXA']    = $row['CDCAIXA'];
            $stDadosFixos['CDEMPRESA']  = $row['CDEMPRESA'];
            $stDadosFixos['CDOPERADOR'] = null;
            
          
             $decodedB64 = base64_decode($row['DSARQUIVOXML']);
             $stDadosFixos['DSARQXMLNFCE'] = $decodedB64;
             $stDadosFixos['NMARQUIVOXML'] = $row['NMARQUIVOXML'];
             $stDadosFixos['NRORG']  = $row['NRORG'];
             $stDadosFixos['CDOPERADOR'] = null;
             
             $arrayDados = $this->gravaArqXml($decodedB64,$stDadosFixos,$connection);
                
                
                $erros=[];
                
                $status =($arrayDados["ERROSTATUS"])?'Não importado':'Importado';
                
              if($arrayDados["ERROSTATUS"]){
                  $erro=[];
                  for($i=0; $i<count($arrayDados["DSERRO"]); $i++){
                      $tipoerro=($arrayDados["IDTIPOMSG"][$i]=="E")?"erro":"alerta";
                      $erro["tipo"]=$tipoerro;
                      $erro["descricao"]=$arrayDados["DSERRO"][$i];
                      $erro["notaFiscal"]=$arrayDados["NRNOTAFISCALCE"];
                      $erro["codigoProduto"]=$arrayDados["CDPRODUTO"][$i];
                      $erro["nomeProduto"]=$arrayDados["NMPRODUTO"][$i];
                      $erro["codigoPagamento"]=$arrayDados["CDTIPORECE"][$i];
                      $erro["ajuda"]=$arrayDados["DSAJUDA"][$i];
                      $erros[]=$erro;
                  }
              }
        
            $resultado=["status"=>$status,"errosAlertas"=>$erros];
            
                $response->addDataset(new DataSet('response', $resultado));
           }
           else{
                $resultado=["status"=>'Não importado',"errosAlertas"=>'Unidade Não encontrada'];
                    $response->addDataset(new DataSet('response', $resultado));
           }
       }
    
    const DEFAULT_XML = '
        <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
        </NFe>';
}